<?php
// $Id: meetu.profile,v 1.2 2010/01/20 00:48:11 JohnKarahalis Exp $

/**
 * @file
 * Core setup tasks of the meetü Installation Profile.
 * 
 * @author
 * John Karahalis (john.karahalis@rit.edu)
 */

/**
 * Username of the account that the Mission Card Scanner will log in with via
 * Services.
 */
define('MEETU_SCANNER_USER', 'GC Scanner');

/**
 * Role for the account that will be used by the Mission Card Scanner.
 */
define('MEETU_SCANNER_ROLE', 'data provider');

/**
 * Implementation of hook_profile_details().
 * Provide a name and description of this profile for the Drupal installer.
 */
function meetu_profile_details() {
  $details = array(
    'name'        => 'meetü',
    'description' => 'meetü is a Drupal-based platform for running games that encourage people to connect with one another, either for an event or as part of an organization’s ongoing effort to get to know itself.',
  );

  return $details;
}

/**
 * Implementation of hook_profile_modules().
 * Return an array of modules to be enabled.
 */
function meetu_profile_modules() {
  $modules = array(

    // Core - required.
    'block',
    'filter',
    'node',
    'system',
    'user',

    // Core - optional.
    'color',
    'dblog',
    'help',
    'menu',
    'path',
    'php',
    'update',

    // CCK.
    'computed_field',
    'content',
    'content_permissions',
    'content_profile',
    'content_profile_registration',
    'fieldgroup',
    'link',
    'nodereference',
    'nodereferrer',
    'number',
    'optionwidgets',
    'text',

    // Date.
    'date_api',       // Dependency of other Date modules.
    'date_timezone',  // Dependency of other Date modules.
    'date',

    // Services.
    'services',
    'node_service',
    'user_service',
    'xmlrpc_server',

    // Views.
    'views',
    'views_bulk_operations',
    'views_customfield',
    'views_ui',
    'views_xml',

    // OPL modules.
    'covert_fields',
    'meetuset',
    'qrs_sheets',
    'qrs_sheet_bulk_render',
    'qrs_sheet_generator',
    'xor_encryption',

    // Other.
    'adminrole',
    'autoassignrole',
    'auto_nodetitle',
    'features',
    'install_profile_api',
    'mobile_codes',
    'pathauto',
    'token',

  );

  return $modules;
}

/**
 * Return an array of Features that should be enabled when this profile is
 * installed.
 *
 * @return
 *   An array of Features to enable.
 */
function meetu_features() {
  $features = array(
    'meetu_feature_core',
    'meetu_feature_p2e',
    'meetu_feature_p2p',
  );

  return $features;
}

/**
 * Implementation of hook_profile_tasks().
 * Handle some tasks and settings that Features does not take care of.
 */
function meetu_profile_tasks(&$task, $url) {

  if($task == 'profile') {
    // Install Features as part of a batch process.
    $modules = meetu_features();
    $files = module_rebuild_cache();
    $operations = array();
    foreach ($modules as $module) {
      $operations[] = array('_install_module_batch', array($module, $files[$module]->info['name']));
    }
    $batch = array(
      'operations' => $operations,
      'finished' => '_meetu_profile_batch_finished',
      'title' => st('Installing @profile', array('@profile' => drupal_install_profile_name())),
      'error_message' => st('The installation has encountered an error.'),
    );
    variable_set('install_task', 'profile-install-batch');
    batch_set($batch);
    batch_process($url, $url);
  }

  if($task == 'meetu-configure') {
    // Enable the Install Profile API.
    install_include(meetu_profile_modules());

    // Set up everything a basic Drupal install provides.
    require_once('profiles/default/default.profile');
    default_profile_tasks($task, $url);

    // services: Don't use keys.
    variable_set('services_use_key', 0);

    // Clear caches. Required for Features permissions to load correctly.
    drupal_flush_all_caches();

    // Handle more involved setup tasks.
    _meetu_setup_content_types();
    _meetu_setup_users();
    _meetu_setup_menu();
    _meetu_setup_auto_nodetitle();
    _meetu_setup_pathauto();

    /* Hide status messages generated by installed modules. These notices are
       not very useful to the person installing, so can be abstracted away. */
    _meetu_hide_status_messages();

    $task = 'finished';
  }
}


/**
 * Callback for finished Features batch install.
 * Set the next installation task.
 */
function _meetu_profile_batch_finished($success, $results) {
  variable_set('install_task', 'meetu-configure');
}

/**
 * Configure settings related to content types.
 */
function _meetu_setup_content_types() {
  // Set up workflow options for content types.
  variable_set('node_options_exhibit', array('status'));
  variable_set('node_options_achievement_p2e', array('status'));
  variable_set('node_options_mission_p2e',     array('status'));
  variable_set('node_options_achievement_p2p', array('status'));
  variable_set('node_options_mission_p2p',     array('status'));
  variable_set('node_options_profile',         array('status'));

  // Content profile settings for the "Profile" content type.
  variable_set('content_profile_use_profile', 1);
  $content_profile_options = variable_get('content_profile_profile', array());
  $content_profile_options['add_link'] = 0;
  $content_profile_options['registration_use'] = 1;
  variable_set('content_profile_profile', $content_profile_options);

  // Set up custom defaults for QR code fields.
  variable_set(QRS_FIELD_HEADER . '_' . QRS_CONTENT_TYPE,  'After the game, register online at [site-url]');
  variable_set(QRS_FIELD_FOOTER . '_' . QRS_CONTENT_TYPE,  '[site-name]');
  variable_set(QRS_FIELD_CONTENT . '_' . QRS_CONTENT_TYPE, '[field_code_shared-raw]');
  variable_set(QRS_FIELD_LABEL . '_' . QRS_CONTENT_TYPE,   '[field_code_shared-raw]');
  variable_set(QRS_FIELD_FIRST_DIFFERENT . '_' . QRS_CONTENT_TYPE, 1);

  // Set up custom defaults for the QR code fieldset relating to the first sticker.
  $qrs_first_sticker = array(
    QRS_FIELD_LABEL => st("Keep this\nto register\nafter the\ngame!\n\nSheet Code:\n[field_code_player-raw]"),
    QRS_FIELD_FIRST_DIFFERENT => TRUE,
  );
  $qrs_first_sticker = serialize($qrs_first_sticker);
  variable_set(QRS_FIELD_FIRST_STICKER . '_' . QRS_CONTENT_TYPE, $qrs_first_sticker);
}

/**
 * Configure settings related to user management.
 */
function _meetu_setup_users() {
  // Get RIDs of some roles.
  $rid_admin  = install_get_rid('administrator');
  $rid_player = install_get_rid('player');
  $rid_data   = install_get_rid(MEETU_SCANNER_ROLE);

  // Grant user 1 (the "root" user) the "administrator" role.
  install_add_user_to_role(1, $rid_admin);

  // Create the scanner user.
  // @TODO: Try to change this process so that the user has control over this account.
  $scanner_pass = user_password(); // Defined here because user_password() is not in scope when we define constants.
  $root_user = user_load(1);
  install_add_user(MEETU_SCANNER_USER, $scanner_pass, $root_user->mail, array(MEETU_SCANNER_ROLE));
  drupal_set_message(st('User "!scanner_user" was created with password "!scanner_pass". These credentials will be used by the game card scanner, so it might be helpful to write them down. To change the username or password of this account at any time, visit the <a href="!user_page">User Administration Page</a>.', array('!scanner_user' => MEETU_SCANNER_USER, '!scanner_pass' => $scanner_pass, '!user_page' => url('admin/user/user', array('absolute' => TRUE)))), 'warning');

  // autoassignrole: Enable automatic role assignment, with "player" as the automatic role.
  module_load_include('inc', 'autoassignrole', 'autoassignrole-admin');
  $form_state = array();
  $form_state['values']['auto_active'] = 1; // Enable automatic assignment.
  $form_state['values']['auto_roles']  = array($rid_player => $rid_player); // Use "player" as automatic assignment.
  drupal_execute('autoassignrole_admin_form', $form_state);

  // adminrole: Make the "administrator" role the adminrole and assign all initial permissions.
  variable_set('user_admin_role', $rid_admin);
  adminrole_update_permissions();

  // Do not require email verification for new users.
  variable_set('user_email_verification', 0);
}

/**
 * Modify the site menu.
 */
function _meetu_setup_menu() {
  // Move "Generate QR-code Sheets" and "Render QR-code Sheets" below the "Sticker sheets" menu item.
  $menu_stickersheets = install_menu_get_items('admin/content/stickersheets');
  $menu_qrs_render    = install_menu_get_items('admin/content/render-all-qrs-sheets');
  $menu_qrs_generate  = install_menu_get_items('admin/content/generate/qrs-sheets');
  install_menu_set_menu($menu_qrs_render[0]['mlid'], $menu_stickersheets[0]['mlid']);
  install_menu_set_menu($menu_qrs_generate[0]['mlid'], $menu_stickersheets[0]['mlid']);
}

/**
 * Configure the auto_nodetitle module to set up automatic node titles by
 * content type.
 */
function _meetu_setup_auto_nodetitle() {
  // Player-to-Exhibit Achievement.
  variable_set('ant_achievement_p2e', 1);
  $ant_pattern_achievement_p2e = st('Sheet ID !player-nid - Achievement for Player-to-Exhibit Mission: !mission-title', array('!player-nid' => '#[field_player-nid]', '!mission-title' => '[field_mission_p2e-title]'));
  variable_set('ant_pattern_achievement_p2e', $ant_pattern_achievement_p2e);  

  // Player-to-Player Achievement.
  variable_set('ant_achievement_p2p', 1);
  $ant_pattern_achievement_p2p = st('Sheet ID !player-nid - Achievement for Player-to-Player Mission: !mission-title', array('!player-nid' => '#[field_player-nid]', '!mission-title' => '[field_mission_p2p-title]'));
  variable_set('ant_pattern_achievement_p2p', $ant_pattern_achievement_p2p);

  // Profile.
  variable_set('ant_profile', 1);
  $ant_pattern_profile = st('User Profile - !author-name', array('!author-name' => '[author-name]'));
  variable_set('ant_pattern_profile', $ant_pattern_profile);

  // QR-Code Sticker Sheet.
  variable_set('ant_qr_sticker_sheet', 1);
  $ant_pattern_qr_sticker_sheet = st('Sticker Sheet - Public ID !code-shared', array('!code-shared' => '#[field_code_shared-raw]'));
  variable_set('ant_pattern_qr_sticker_sheet', $ant_pattern_qr_sticker_sheet);
}

/**
 * Configure the pathauto module.
 */
function _meetu_setup_pathauto() {
  // pathauto: Automatic alias settings.
  variable_set('pathauto_node_pattern',                  'content/[title-raw]');
  variable_set('pathauto_node_exhibit_pattern',          'exhibit/[title-raw]');
  variable_set('pathauto_node_achievement_p2e_pattern',  'achievement/[field_player-nid]/p2e/[field_mission_p2e-title-raw]');
  variable_set('pathauto_node_mission_p2e_pattern',      'mission/p2e/[title-raw]');
  variable_set('pathauto_node_achievement_p2p_pattern',  'achievement/[field_player-nid]/p2p/[field_mission_p2p-title-raw]');
  variable_set('pathauto_node_mission_p2p_pattern',      'mission/p2p/[title-raw]');
  variable_set('pathauto_node_qr_sticker_sheet_pattern', 'sticker-sheet/[field_code_shared-raw]');
  variable_set('pathauto_user_pattern',                  'user/[user-raw]');
}

/**
 * Implementation of hook_form_FORM_ID_alter().
 */
function meetu_form_alter(&$form, $form_state, $form_id) {
  // Provide some defaults on the the "Configure site" page.
  // Gratefully borrowed and adapted from Managing News source - managingnews.com
  if ($form_id == 'install_configure') {
    $default_user  = 'admin';
    $default_email = "$default_user@" . $_SERVER['HTTP_HOST'];

    $form['site_information']['site_name']['#default_value'] = drupal_install_profile_name();
    $form['site_information']['site_mail']['#default_value'] = $default_email;
    $form['admin_account']['account']['name']['#default_value'] = $default_user;
    $form['admin_account']['account']['mail']['#default_value'] = $default_email;
  }
}

/**
 * Remove all status messages currently in the message queue.
 */
function _meetu_hide_status_messages() {
  drupal_get_messages('status');
}
